using System;
using System.Drawing;
using System.Windows.Forms;

/* Weight of FibonacciNodes should be generic.
 * Generics themselves are crappy for this Heap, because Copmarison 
 * operators are assured about the interface IComparable, but + operator
 * does'nt have an interface. Because of that a generic class had to
 * encapsulate a value type and additionally support a user defined
 * interface like "IAddable" or sth.
 * 
 * Instead, the following statement should be sufficient for our needs. */
using TWeight = System.Decimal;

public delegate void OnApplyHandler(TWeight[,] m);

public class MatrixControl : Panel
{
	// Members
	public TWeight[,] pathContainer;
	private Cell[][] Cells;
	public event OnApplyHandler OnApply;

	// C'tor
	public MatrixControl(int size)
	{
		this.pathContainer = new TWeight[size, size];

		for (int i=0; i<size; i++)
			for (int ii=0; ii<size; ii++)
				this.pathContainer[i, ii] = -1;
		
		// Labels
		Index tmp;
		int margin = 0;

		for (int i=0; i<size; i++)
		{
			tmp = new Index();

			if (i == 0)
				margin = tmp.Height + 1;

			tmp.Text = i.ToString();
			tmp.Left = i * (tmp.Width + 1) + 1;
			tmp.Top = 1;
			this.Controls.Add(tmp);
		}

		for (int i=0; i<size; i++)
		{
			tmp = new Index();

			tmp.Text = i.ToString();
			tmp.BorderStyle = BorderStyle.FixedSingle;
			tmp.IsAccessible = false;
			tmp.BackColor = Color.LightSteelBlue;
			tmp.Width = tmp.Height;
			tmp.Left = size * (tmp.Width + 1) + 1;
			tmp.Top = i * (tmp.Width + 1) + 1 + tmp.Width + 1;
			this.Controls.Add(tmp);
		}

		tmp = new Index();
		tmp.Text = "Start / Dest";
		tmp.Left = size * (tmp.Width + 1) + 1;
		tmp.Top = 1;
		tmp.Width = 80;
		this.Controls.Add(tmp);

		// Cells
		this.Cells = new Cell[size][];

		for (int i=0; i<size; i++)
			this.Cells[i] = new Cell[size];

		for (int i=0; i<size; i++)
			for (int ii=0; ii<size; ii++)
			{
				this.Cells[i][ii] = new Cell();
				this.Cells[i][ii].Left = i * (this.Cells[i][ii].Width + 1) + 1;
				this.Cells[i][ii].Top = ii * (this.Cells[i][ii].Height + 1) + 1 + margin;
				this.Controls.Add(this.Cells[i][ii]);
			}

		Button bApply = new Button();
		// bApply.BackColor = Color.Red;		// testing
		bApply.Left = (size + 1) * (this.Cells[0][0].Width + 1) + 1;
		bApply.Top = (size - 1)* (this.Cells[0][0].Width + 1) + 1;
		bApply.Width = 80 - this.Cells[0][0].Width - 2;
		bApply.Height = this.Cells[0][0].Width * 2 + 1;
		bApply.Text = "Apply";
		bApply.Click += new EventHandler(ApplyNewPath);
		this.Controls.Add(bApply);

		this.Width = (this.Cells[0][0].Height + 1) * size + margin + 85;
		this.Height = this.Width - 80;
	}

	private void ApplyNewPath(object sender, EventArgs e)
	{
		for (int i=0; i<this.Cells.Length; i++)
			for (int ii=0; ii<this.Cells.Length; ii++)
				this.pathContainer[i, ii] = (TWeight)Int32.Parse(this.Cells[i][ii].Text);

		if (this.OnApply != null)
			this.OnApply(this.pathContainer);
	}

	protected override void OnPaint(PaintEventArgs e)
	{
	
	}
}

public class Cell : TextBox
{
	public Cell()
	{
		this.Text = "-1";
		this.BorderStyle = BorderStyle.FixedSingle;
		this.Width = this.Height;

	}
}

public class Index : TextBox
{
	public Index()
	{
		this.BorderStyle = BorderStyle.FixedSingle;
		this.BackColor = Color.LightSteelBlue;
		this.Width = this.Height;
	}
}