using System;
using System.Drawing;
using System.Windows.Forms;

/* Weight of FibonacciNodes should be generic.
 * Generics themselves are crappy for this Heap, because Copmarison 
 * operators are assured about the interface IComparable, but + operator
 * does'nt have an interface. Because of that a generic class had to
 * encapsulate a value type and additionally support a user defined
 * interface like "IAddable" or sth.
 * 
 * Instead, the following statement should be sufficient for our needs. */
using TWeight = System.Decimal;

public class View : Panel
{
	// Members
	private TWeight[,] m;
	private Point[] p;
	private int radius = 15;

	// C'tor
	public View()
	{
		this.p = new Point[16];

		this.p[0] = new Point(5, 5 + 7 * radius);

		this.p[1] = new Point(radius * 2 + 2 * 5, 5 + 2 * radius);
		this.p[2] = new Point(radius * 2 + 2 * 5, 5 + 7 * radius);
		this.p[3] = new Point(radius * 2 + 2 * 5, 5 + 12 * radius);

		this.p[4] = new Point(radius * 4 + 4 * 5, 5);
		this.p[5] = new Point(radius * 4 + 4 * 5, 5 + 5 * radius);
		this.p[6] = new Point(radius * 4 + 4 * 5, 5 + 10 * radius);
		this.p[7] = new Point(radius * 4 + 4 * 5, 5 + 15 * radius);

		this.p[8] = new Point(radius * 6 + 6 * 5, 5);
		this.p[9] = new Point(radius * 6 + 6 * 5, 5 + 5 * radius);
		this.p[10] = new Point(radius * 6 + 6 * 5, 5 + 10 * radius);
		this.p[11] = new Point(radius * 6 + 6 * 5, 5 + 15 * radius);

		this.p[12] = new Point(radius * 8 + 8 * 5, 5 + 2 * radius);
		this.p[13] = new Point(radius * 8 + 8 * 5, 5 + 7 * radius);
		this.p[14] = new Point(radius * 8 + 8 * 5, 5 + 12 * radius);

		this.p[15] = new Point(radius * 10 + 10 * 5, 5 + 7 * radius);
	}

	// Methods
	public void ShowPathNetwork(TWeight[,] m)
	{
		this.m = m;
		this.Invalidate();
	}

	protected override void OnPaint(PaintEventArgs e)
	{
		base.OnPaint(e);

		// Show nodes
		Pen p = new Pen(Color.Black);

		e.Graphics.DrawEllipse(p, 5, 5 + 7 * radius, radius, radius);

		e.Graphics.DrawEllipse(p, radius * 2 + 2 * 5, 5 + 2 * radius, radius, radius);
		e.Graphics.DrawEllipse(p, radius * 2 + 2 * 5, 5 + 7 * radius, radius, radius);
		e.Graphics.DrawEllipse(p, radius * 2 + 2 * 5, 5 + 12 * radius, radius, radius);

		e.Graphics.DrawEllipse(p, radius * 4 + 4 * 5, 5, radius, radius);
		e.Graphics.DrawEllipse(p, radius * 4 + 4 * 5, 5 + 5 * radius, radius, radius);
		e.Graphics.DrawEllipse(p, radius * 4 + 4 * 5, 5 + 10 * radius, radius, radius);
		e.Graphics.DrawEllipse(p, radius * 4 + 4 * 5, 5 + 15 * radius, radius, radius);

		e.Graphics.DrawEllipse(p, radius * 6 + 6 * 5, 5, radius, radius);
		e.Graphics.DrawEllipse(p, radius * 6 + 6 * 5, 5 + 5 * radius, radius, radius);
		e.Graphics.DrawEllipse(p, radius * 6 + 6 * 5, 5 + 10 * radius, radius, radius);
		e.Graphics.DrawEllipse(p, radius * 6 + 6 * 5, 5 + 15 * radius, radius, radius);

		e.Graphics.DrawEllipse(p, radius * 8 + 8 * 5, 5 + 2 * radius, radius, radius);
		e.Graphics.DrawEllipse(p, radius * 8 + 8 * 5, 5 + 7 * radius, radius, radius);
		e.Graphics.DrawEllipse(p, radius * 8 + 8 * 5, 5 + 12 * radius, radius, radius);

		e.Graphics.DrawEllipse(p, radius * 10 + 10 * 5, 5 + 7 * radius, radius, radius);

		// Show inndices
		for (int i=0; i<this.p.Length; i++)
			e.Graphics.DrawString(i.ToString(), this.Font, Brushes.Black, (PointF)this.p[i]);
		
		if (this.m == null)
			return;

		// Show paths
		for (int i=0; i<m.GetLength(0); i++)			// for each startNode
		{
			for (int ii=0; ii<m.GetLength(0); ii++)		// check connection to any destinationNode
			{											// 2Do: check bidirectionality
				if (m[ii, i] != -1)
					e.Graphics.DrawLine(p, this.p[i], this.p[ii]);
			}
		}

	}
}
